#
# This file is part of Zaplet
# Copyright 1999 Adam Feuer <adamf@pobox.com>
#
# Zaplet is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Zaplet is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Zaplet; see the file COPYING-Zaplet. If not, write to the
# Free Software Foundation, Inc.,
# 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
#
#
# adzapper_config.py
# to read the adzapper.conf file and set adzapper defaults
#
# for use with adzapper filtering proxy

RCS_ID =  '$Id: adzapper_engine.py,v 1.3 1999/09/23 05:12:22 adamf Exp $'

# constants
ALLOWDENY = 0
DENYALLOW = 1

import os
import re
import string
import os
import sys
import urlparse
import socket

# adzapper modules
import debug
import blank_gif

numeric_host = re.compile ('[0-9]+\.[0-9]+\.[0-9]+\.[0-9]+')
numeric_network = re.compile ('[0-9]+\.([0-9]+\.)?([0-9]+\.)?([[0-9]+\*])?')


class adzapper_engine:
    """initialize the adzapper_engine object; set all defaults"""

    def __init__(self, adzapper_conf_file):
        self.adzapperConfigFile = adzapper_conf_file

        # access control list - lists of IPs we should allow or deny
        # defaults to off
        self.acOn = 0
        self.aclOrder = DENYALLOW  # defaults to deny all, then allow specific hosts and networks
        self.aclAllow = []
        self.aclDeny = []
        self.aclAllow_re = None
        self.aclDeny_re = None

        # replacement gif
        self.replacement_gif = blank_gif.default_gif

        # HTTP proxy - default is None
        self.http_proxy = None

        # adzapper.conf version defaults to 0.1
        self.majorversion = 0  
        self.minorversion = 1

        self.read_config_file(self.adzapperConfigFile)


    def read_config_file(self,adzapperConfigFile):
        """read the adzapper.conf file
        this file contains global settings for adzapper such as access control, etc.
        note: zDir must have a trailing '/' """

        debug.debug(1,"Initializing adzapper_engine...")

        if (adzapperConfigFile[0] == '/'):
            # it is an absolute path
            azengcfg = adzapperConfigFile
        else:
            # not absolute path to the adzapper.conf file-- add cwd to it
            azengcfg = os.path.join(os.getcwd(),adzapperConfigFile)

        # read the adzapper.conf file
        # this will hold configuration options for adzapper as a whole
        try:
            debug.debug(1," opening adzapper.conf  file: " + azengcfg)
            cf = open(azengcfg, "r")
        except:
            sys.stderr.write("Cannot open adzapper.conf file "+ azengcfg +"\n")
            sys.exit(0)

        lines = cf.readlines()
        cf.close()
        for line in lines:
            line = string.strip(line)
            debug.debug(1," line: " + line)
            if (len(line) == 0) or re.search("^\s*#", line):
                continue
            if string.find(line, '#'):
                ltmp = string.split(line, '#')
                line = ltmp[0]
            tokens = re.split('\s+', line)
            command = string.lower(tokens[0])
            if (command == 'version'):
                vstr = tokens[1]
                vlist = string.split(vstr,'.')
                self.zconf_majorversion = string.atoi(vlist[0])
                self.zconf_minorversion = string.atoi(vlist[1])
                debug.debug(1,'adzapper.conf version: %i.%i' %
                            (self.majorversion, self.minorversion))

            elif (command == 'order'):
                arg = tokens[1]
                self.setaclOrder(arg)
            elif (command == 'allow'):
                args = tokens[1:]
                self.addaclAllow(args)
            elif (command == 'deny'):
                args = tokens[1:]
                self.addaclDeny(args)
            elif (command == 'replacement_gif'):
                arg = tokens[1]
                self.read_replacement_gif(arg)
            elif (command == 'http_proxy'):
                arg = tokens[1]
                self.set_http_proxy(arg)
            elif (command == 'pid_file'):
                arg = tokens[1]
                self.set_pid_file(arg)
            else:
                sys.stderr.write("Bad configuration line: "+line+"\n")

        # end for


        # initialize the access control
        if len(self.aclAllow) > 0:
            self.acOn = 1
            reStr = ''
            debug.debug(4,"init acl: %s" % self.aclAllow)
            for arg in self.aclAllow:
                reStr = reStr + '|' + arg
            # chop off initial '|'
            reStr = reStr[1:]
            debug.debug(4, "adzapper acl allow: %s" % reStr)
            self.aclAllow_re = re.compile(reStr)

        if len(self.aclDeny) > 0:
            self.acOn = 1
            reStr = ''
            debug.debug(4,"init acl: %s" % self.aclDeny)
            for arg in self.aclDeny:
                reStr = reStr + '|' + arg
            # chop off initial '|'
            reStr = reStr[1:]
            debug.debug(4, "adzapper acl deny: %s" % reStr)
            self.aclDeny_re = re.compile(reStr)                        



    def read_replacement_gif(self,file):
        """read in the user-supplied replacement gif"""

        if (file[0] == '/'):
            # it is an absolute path
            pass
        else:
            # not absolute path to file-- add cwd to it
            file = os.path.join(os.getcwd(),file)

        debug.debug(4,"replacement gif filename: %s " % file)
        tmplist = os.stat(file)
        self.replacement_gif.length = tmplist[6]
        file = open(file, 'r')
        self.replacement_gif.data = file.read(self.replacement_gif.length)
        file.close()



    def set_http_proxy(self,http_proxy_url):
        """set the HTTP proxy url
        (this is for setting adzapper to use a proxy server)"""
        self.http_proxy = 1
        self.http_proxy_url = http_proxy_url

        url_tuple = urlparse.urlparse(http_proxy_url)
        netloc = url_tuple[1]
        if ':' in netloc:
            (host,port) = string.split(netloc,':')
        else:
            host = netloc
            port = '8080'

        self.http_proxy_ip = socket.gethostbyname(host)

        if len(port) == 0:
            port = '8080'

        self.http_proxy_port = string.atoi(port)

        debug.debug(3,"http proxy server: %s" % self.http_proxy_url)
        debug.debug(3,"http proxy ip, port: %s" % self.http_proxy_ip)
        debug.debug(3,"http proxy ip, port: %d" % self.http_proxy_port)


    def set_pid_file(self,file):
        """set the pid file; write the pid to this file"""

        self.pid_file = file
        debug.debug(4,"pid filename: %s " % self.pid_file)
        self.pid = "%d" % os.getpid()
        debug.debug(4,"pid: %s " % self.pid)
        file = open(file, 'w')
        file.write(self.pid)
        file.close()


    def setaclOrder(self, arg=None):
        """set the access control order by interpreting the arguments"""
        
        if arg == None:
            debug.debug(1,"syntax error: need either allow,deny or deny,allow keywords.")

        if (arg == 'allow,deny'):
            self.aclOrder = ALLOWDENY
            return
        if (arg == 'deny,allow'):
            self.aclOrder = DENYALLOW
            return
        debug.debug(1,"syntax error: order must be followed by \"allow,deny\" or \"deny,allow\"")


    def addaclAllow(self,args):
        """add an access control regex to the Allow acl
        interpret allow strings and build regexes for them"""
        
        for arg in args:
            debug.debug(1,"addacl- arg: %s" % arg)
            if numeric_host.match(arg):
                debug.debug(1,"addacl- numeric host: %s" % arg)
                # it's a literal string, so we need to escape it.
                arg = re.escape(arg)
                self.aclAllow.append(arg)
            elif numeric_network.match(arg):
                debug.debug(1,"addacl- numeric network: %s" % arg)
                # if there is a '*' at the end, get rid of it
                if arg[len(arg)-1] == '*':
                    arg = arg[0:len(arg)-1]
                    debug.debug(1,"addacl- numeric network (chopped): %s" % arg)
                # it's a literal string, so we need to escape it.
                arg = re.escape(arg)
                self.aclAllow.append(arg)
            else:
                debug.debug(1,"addacl- no match: %s" % arg)
                if arg == 'all':
                    self.aclAllow.append(arg)
                elif arg == 'from':
                    pass # allow apache-like syntax
                debug.debug(1,"addacl- literal: %s" % arg)
        if 'all' in self.aclAllow:
            debug.debug(4,"addacl: found all")
            self.aclAllow = ['.*'] # match any IP address


    def addaclDeny(self,args):
        """add an access control regex to the Deny acl
        interpret allow strings and build regexes for them"""
    
        for arg in args:
            debug.debug(1, "addacl- arg: %s" % arg)
            if numeric_host.match(arg):
                debug.debug(1,"addacl- numeric host: %s" % arg)
                self.aclDeny.append(arg)
            elif numeric_network.match(arg):
                debug.debug(1,"addacl- numeric network: %s" % arg)
                # if there is a '*' at the end, get rid of it
                if arg[len(arg)-1] == '*':
                    arg = arg[0:len(arg)-1]
                debug.debug(1,"addacl- numeric network (chopped): %s" % arg)
                # it's a literal string, so we need to escape it.
                arg = re.escape(arg)
                self.aclDeny.append(arg)
            else:
                if arg == 'all':
                    self.aclDeny.append(arg)
                elif arg == 'from':
                    pass # allow apache-like syntax
                debug.debug(1,"addacl- literal: %s" % arg)
        if 'all' in self.aclDeny:
            debug.debug(4, "addacl: found all")
            self.aclDeny = ['.*'] # match any IP address


    def checkAllow(self,requesterIP):
        """return 1 if requesterIP matches aclAllow_re
        no aclAllow means nothing matches == nothing allowed"""

        if self.aclAllow_re != None:
            if self.aclAllow_re.match(requesterIP):
                return 1
        return 0

    def checkDeny(self,requesterIP):
        """return 1 if requesterIP matches aclDeny_re
        no aclDeny means everything matches == everything denied"""
        
        if self.aclDeny_re != None:
            if self.aclDeny_re.match(requesterIP):
                return 1
            else:
                return 0
        return 1


    def accesscontroller(self,requesterIP):
        """determine whether a particular IP address should be allowed to use the adzapper
        returns 1 if an IP address should be blocked, 0 otherwise"""
        
        debug.debug(4, "requesterIP: [%s]" % requesterIP)
        if self.acOn == 0:
            return 0

        if self.aclOrder == ALLOWDENY:
            # check allow first, then deny
            if self.checkAllow(requesterIP):
                if self.checkDeny(requesterIP):
                    return 1 # denied
                else:
                    return 0 # not denied == allowed
            else:
                return 1 # not allowed == denied

        else: # self.aclOrder == DENYALLOW
            # check deny first, then do denies.
            if self.checkDeny(requesterIP):
                if self.checkAllow(requesterIP):
                    return 0 # allowed
                else:
                    return 1 # not allowed == denied
            else:
                return 0 # not denied == allowed
